# Question Answering using SPA

# Now we will build the model again, using the spa (semantic pointer
# architecture) package built into Nengo 2.0. You will see that using the spa
# package considerably simplifies model construction and visualization through
# nengo_gui.

# Press the play button to run the simulation.
# Graphs show the colour, shape and cue inputs. The last graph
# shows that the output is most similar to the semantic pointer bound to the
# current cue. For example, when RED and CIRCLE are being bound
# and the cue is CIRCLE, the output is most similar to RED.

# Setup the environment
import nengo
import nengo.spa as spa
import numpy as np
from nengo.spa import Vocabulary

dim = 32  # The dimensionality of the vectors
rng = np.random.RandomState(4)
vocab = Vocabulary(dimensions=dim, rng=rng, max_similarity=0.1)

# Adding semantic pointers to the vocabulary
CIRCLE = vocab.parse("CIRCLE")
BLUE = vocab.parse("BLUE")
RED = vocab.parse("RED")
SQUARE = vocab.parse("SQUARE")
ZERO = vocab.add("ZERO", [0] * dim)

# Create the spa.SPA network to which we can add SPA objects
model = spa.SPA(label="Question Answering", vocabs=[vocab])
with model:
    model.A = spa.State(dim)
    model.B = spa.State(dim)
    model.C = spa.State(dim)
    model.D = spa.State(dim)
    model.E = spa.State(dim)

    actions = spa.Actions(
        "D = A * B",
        "E = D * ~C",
    )

    model.cortical = spa.Cortical(actions)

    # Function for providing color input
    def color_input(t):
        if (t // 0.5) % 2 == 0:
            return "RED"
        return "BLUE"

    # Function for providing shape input
    def shape_input(t):
        if (t // 0.5) % 2 == 0:
            return "CIRCLE"
        return "SQUARE"

    # Function for providing the cue
    def cue_input(t):
        sequence = ["ZERO", "CIRCLE", "RED", "ZERO", "SQUARE", "BLUE"]
        idx = int((t // (1.0 / len(sequence))) % len(sequence))
        return sequence[idx]

    # Inputs
    model.input = spa.Input(A=color_input, B=shape_input, C=cue_input)



from __future__ import absolute_import

import collections.abc
import sys

import numpy as np

# Only test for Python 2 so that we have less changes for Python 4
PY2 = sys.version_info[0] == 2

# If something's changed from Python 2 to 3, we handle that here
if PY2:
    from cgi import escape as cgi_escape

    from StringIO import StringIO

    escape = lambda s, quote=True: cgi_escape(s, quote=quote)
    iteritems = lambda d: d.iteritems()

    string_types = (str, unicode)
    int_types = (int, long)
    range = xrange

else:
    from html import escape
    from io import StringIO

    iteritems = lambda d: iter(d.items())


def is_iterable(obj):
    if isinstance(obj, np.ndarray):
        return obj.ndim > 0  # 0-d arrays give error if iterated over
    else:
        return isinstance(obj, collections.abc.Iterable)



# Routed Sequencing

# This model introduces routing in the sequencing model. The addition of routing
# allows the system to choose between two different actions: whether to go
# through the sequence, or be driven by the visual input as explained in the
# book. For instance, if the visual input has its value set to 0.8*START+D, the
# model will begin cycling through at D->E, etc. Thus in this model, the input
# doesn't prevent the activation of the second rule in the sequence.

# The parameters used in the model are as described in the book, with 16
# dimensions for all semantic pointers.

# In Nengo 1.4, a buffer element for representing the vision was created by
# using Buffer() as described in the book. However, in Nengo 2.0, you will have
# to use State() with feedback parameter set to 0 (which is the default value in
# nengo).

# Press the play button in the visualizer to run the simulation.
# The graph on the bottom-left shows the visual input recieved by the model,
# the state graph in the middle shows the semantic pointer representation of the
# values stored in the state ensemble. The actions graph on bottom-right shows
# the current transition or the action being executed, and the state graph on
# top-right shows the utility (similarity) of the current basal ganglia input
# (i.e., state) with the possible vocabulary vectors.

# You can see that in this model, even though the input is applied for 400ms, it
# doesn't prevent the activation of the second and subsequent rules in the
# sequence.

# Setup the environment
import nengo
from nengo import spa  # import spa related packages

# Number of dimensions for the semantic pointers
dim = 16

# Create the spa.SPA network to which we can add SPA objects
model = spa.SPA(label="Routed_Sequence", seed=20)
with model:
    # Specify the modules to be used
    model.state = spa.State(dimensions=dim, feedback=1, feedback_synapse=0.01)
    model.vision = spa.State(dimensions=dim)

    # Specify the action mapping
    actions = spa.Actions(
        "dot(vision, START) --> state = vision",
        "dot(state, A) --> state = B",
        "dot(state, B) --> state = C",
        "dot(state, C) --> state = D",
        "dot(state, D) --> state = E",
        "dot(state, E) --> state = A",
    )

    # Creating the BG and thalamus components that confirm to the specified rules
    model.bg = spa.BasalGanglia(actions=actions)
    model.thal = spa.Thalamus(model.bg)

    # Function that provides the model with an initial input semantic pointer.
    def start(t):
        if t < 0.4:
            return "0.8*START+D"
        return "0"

    # Input
    model.input = spa.Input(vision=start)



import re

try:
    from io import BytesIO
except ImportError:  # Python 2.7
    from BytesIO import BytesIO

from nengo_gui import server


class TwoWayStringIO(object):
    def __init__(self, readable_data):
        self.readable_data = readable_data
        self.read_buffer = BytesIO(self.readable_data)
        self.write_buffer = BytesIO()
        self.written_data = None
        self.closed = False

    def read(self, *args, **kwargs):
        return self.read_buffer.read(*args, **kwargs)

    def readline(self, *args, **kwargs):
        return self.read_buffer.readline(*args, **kwargs)

    def write(self, *args, **kwargs):
        return self.write_buffer.write(*args, **kwargs)

    def close(self):
        self.closed = True
        self.written_data = self.write_buffer.getvalue()
        self.read_buffer.close()

    def flush(self):
        self.write_buffer.flush()


class ServerMock(object):
    server_name = "localhost"
    server_port = 80

    def create_websocket(self, socket):
        return server.WebSocket(socket)


class SocketMock(object):
    def __init__(self, request_data):
        self.request_data = request_data.encode("utf-8")
        self.file_io = TwoWayStringIO(self.request_data)

    def makefile(self, mode="r", bufsize=1024):
        return self.file_io

    def send(self, data):
        self.file_io.write(data)
        return len(data)

    def sendall(self, data):
        self.send(data)

    def setblocking(self, value):
        pass


class TestHttpWsRequestHandler(object):
    def test_get(self):
        request = SocketMock(
            """GET / HTTP/1.1
Host: localhost
User-Agent: nengo_gui
"""
        )

        class HandlerClass(server.HttpWsRequestHandler):
            def __init__(self, request, client_address, srv):
                self.method_called = False
                server.HttpWsRequestHandler.__init__(self, request, client_address, srv)

            def http_GET(self):
                self.method_called = True
                assert self.path == "/"
                assert self.headers.get("host").lower() == "localhost"
                assert self.headers.get("user-agent").lower() == "nengo_gui"

        handler = HandlerClass(request, "localhost", ServerMock())
        assert handler.method_called

    def test_upgrade_to_websocket(self):
        request = SocketMock(
            """GET /resource_name HTTP/1.1
Upgrade: websocket
Connection: Upgrade
Origin: http://localhost:80
Host: localhost:80
Sec-WebSocket-Key: AQIDBAUGBwgJCgsMDQ4PEC==
Sec-WebSocket-Version: 13
"""
        )

        class HandlerClass(server.HttpWsRequestHandler):
            def ws_default(self):
                pass

            def get_expected_origins(self):
                return ["localhost:80"]

        handler = HandlerClass(request, "localhost", ServerMock())
        assert handler
        # Ignore errors in decoding because binary WebSocket data might follow
        # response.
        response = request.file_io.written_data.decode("utf-8", errors="ignore")

        assert re.match(r"^HTTP\/\d+\.\d+\s+101.*$", response, re.M)
        assert re.search(r"^Upgrade:\s+.*websocket.*$", response, re.M | re.I)
        assert re.search(r"^Connection:\s+.*upgrade.*$", response, re.M | re.I)
        assert re.search(
            r"^Sec-WebSocket-Accept:\s+OfS0wDaT5NoxF2gqm7Zj2YtetzM=\s+$", response, re.M
        )

    def test_bad_upgrade_to_websocket(self):
        request = SocketMock(
            """GET /resource_name HTTP/1.1
Upgrade: websocket
Connection: Upgrade
Origin: http://localhost:80
Host: localhost:80
Sec-WebSocket-Key: null
Sec-WebSocket-Version: 13
"""
        )

        class HandlerClass(server.HttpWsRequestHandler):
            def get_expected_origins(self):
                return ["localhost:80"]

        handler = HandlerClass(request, "localhost", ServerMock())
        assert handler
        response = request.file_io.written_data.decode("utf-8")

        assert re.match(r"^HTTP\/\d+\.\d+\s+400.*$", response, re.M)

    def test_parsing_resource_get(self):
        request = SocketMock("GET /res/file?p1=1&p2=2&p1=0 HTTP/1.1\r\n")

        class HandlerClass(server.HttpWsRequestHandler):
            def http_default(self):
                pass

        handler = HandlerClass(request, "localhost", ServerMock())
        assert handler.resource == "/res/file"
        assert handler.query == {"p1": ["1", "0"], "p2": ["2"]}
        assert handler.db == {"p1": "1", "p2": "2"}

    def test_parsing_resource_post(self):
        content = "p2=3&p3=0"
        request = SocketMock(
            """POST /res/file?p1=1&p2=2&p1=0 HTTP/1.1
Content-Type:application/x-www-form-urlencoded; charset=UTF-8
Content-Length: {len}

{content}
""".format(
                len=len(content), content=content
            )
        )

        class HandlerClass(server.HttpWsRequestHandler):
            def http_default(self):
                pass

        handler = HandlerClass(request, "localhost", ServerMock())
        assert handler.resource == "/res/file"
        assert handler.query == {"p1": ["1", "0"], "p2": ["2"]}
        assert handler.db == {"p1": "1", "p2": "3", "p3": "0"}

    def test_http_dispatch(self):
        request = SocketMock("GET /file HTTP/1.1\r\n")

        class HandlerClass(server.HttpWsRequestHandler):
            called = False
            http_commands = {"/file": "cmd"}

            def cmd(self):
                self.called = True
                return b""

        handler = HandlerClass(request, "localhost", ServerMock())
        assert handler.called

    def test_ws_dispatch(self):
        request = SocketMock(
            """GET /ws HTTP/1.1
Upgrade: websocket
Connection: Upgrade
Origin: http://localhost:80
Host: localhost:80
Sec-WebSocket-Key: AQIDBAUGBwgJCgsMDQ4PEC==
Sec-WebSocket-Version: 13
"""
        )

        class HandlerClass(server.HttpWsRequestHandler):
            called = False
            ws_commands = {"/ws": "cmd"}
            server = ServerMock()

            def cmd(self):
                self.called = True

            def get_expected_origins(self):
                return ["localhost:80"]

        handler = HandlerClass(request, "localhost", ServerMock())
        assert handler.called
        assert handler.ws.state in [
            server.WebSocket.ST_CLOSING,
            server.WebSocket.ST_CLOSED,
        ]



# Question Answering with Memory

# This model shows a form of question answering with memory. It binds two
# features (color and shape) by circular convolution and stores them in a memory
# population. It then provides a cue to the model at a later time to
# determine the feature bound to that cue by deconvolution. This model exhibits
# better cognitive ability since the answers to the questions are provided at a
# later time and not at the same time as the questions themselves.

# **Note: A simplified method of building the model using the spa (semantic
# pointer architecture) package in Nengo 2.0 is shown in the
# spa_question-memory.py file in the same folder.

# This model has parameters as described in the book, with memory population
# having 1000 neurons over 20 dimensions. The memory population is capable of
# storing a vector over time and it uses an integrator network to do so as
# discussed in the book.

# The color input presented to the model is RED and then BLUE for 0.25 seconds
# each before being turned off. In the same way the shape input is CIRCLE and
# then SQUARE for 0.25 seconds each. Thus, when you run the model, it will start
# by binding RED and CIRCLE for 0.25 seconds and then binding BLUE and SQUARE
# for 0.25 seconds. The cue for deconvolving bound semantic pointers will be
# turned off for 0.5 seconds and then starts cycling through CIRCLE, RED,
# SQUARE, and BLUE within each second. The model will be able to determine the
# correct answer using the cue, even when the color and shape inputs have been
# turned off.

# Press the play button to run the simulation.
# The graphs of shape, color, bound and memory show that first RED * CIRCLE and
# then BLUE * SQUARE are convolved (bound) and loaded into the memory population,
# so after 0.5 seconds the memory represents the superposition
# RED * CIRCLE + BLUE * SQUARE. The last plot shows that the output is most
# similar to the semantic pointer bound to the current cue. For example, when
# the cue is CIRCLE, the output is most similar to RED.  Again, this is much
# easier to see in the spa_question-memory.py example.

import nengo

# Setup the environment
import numpy as np
from nengo.spa import Vocabulary

dim = 32  # Number of dimensions
n_neurons = 300  # Number of neurons in population
n_conv = 70  # Number of neurons per dimension in bind/unbind populations
n_mem = 50  # Number of neurons per dimension in memory population

# Creating the vocabulary
rng = np.random.RandomState(0)
vocab = Vocabulary(dimensions=dim, rng=rng, max_similarity=0.1)

model = nengo.Network(label="Question Answering with Memory", seed=12)
with model:
    # Ensembles
    ens_A = nengo.Ensemble(n_neurons=n_neurons, dimensions=dim, label="A")
    ens_B = nengo.Ensemble(n_neurons=n_neurons, dimensions=dim, label="B")
    ens_C = nengo.Ensemble(n_neurons=n_neurons, dimensions=dim, label="C")
    ens_D = nengo.Ensemble(n_neurons=n_neurons, dimensions=dim, label="D")
    ens_E = nengo.Ensemble(n_neurons=n_neurons, dimensions=dim, label="E")

    # Creating memory population and connecting ensemble D to it
    tau = 0.4
    memory = nengo.networks.EnsembleArray(
        n_neurons=n_mem,
        n_ensembles=dim,
        label="Memory",
    )
    nengo.Connection(memory.output, memory.input, synapse=tau)
    nengo.Connection(ens_D, memory.input)

    # Creating the Bind network
    net_bind = nengo.networks.CircularConvolution(n_neurons=n_conv, dimensions=dim)
    nengo.Connection(ens_A, net_bind.A)
    nengo.Connection(ens_B, net_bind.B)
    nengo.Connection(net_bind.output, ens_D)

    # Creating the Unbind network
    net_unbind = nengo.networks.CircularConvolution(
        n_neurons=n_conv, dimensions=dim, invert_a=True
    )
    nengo.Connection(ens_C, net_unbind.A)
    nengo.Connection(memory.output, net_unbind.B)
    nengo.Connection(net_unbind.output, ens_E)

    # Getting semantic pointer values
    CIRCLE = vocab.parse("CIRCLE").v
    BLUE = vocab.parse("BLUE").v
    RED = vocab.parse("RED").v
    SQUARE = vocab.parse("SQUARE").v
    ZERO = [0] * dim

    # Function for providing color input
    def color_input(t):
        if t < 0.25:
            return RED
        elif t < 0.5:
            return BLUE
        return ZERO

    # Function for providing shape input
    def shape_input(t):
        if t < 0.25:
            return CIRCLE
        elif t < 0.5:
            return SQUARE
        return ZERO

    # Function for providing the cue
    def cue_input(t):
        if t < 0.5:
            return ZERO
        sequence = [ZERO, CIRCLE, RED, ZERO, SQUARE, BLUE]
        idx = int(((t - 0.5) // (1.0 / len(sequence))) % len(sequence))
        return sequence[idx]

    # Defining inputs
    input_A = nengo.Node(output=color_input, size_out=dim, label="Input A")
    input_B = nengo.Node(output=shape_input, size_out=dim, label="Input B")
    input_C = nengo.Node(output=cue_input, size_out=dim, label="Input C")

    # Connecting input to ensembles
    nengo.Connection(input_A, ens_A)
    nengo.Connection(input_B, ens_B)
    nengo.Connection(input_C, ens_C)



# Structured Representations

# This model shows a method for constructing structured representations
# using semantic pointers (high-dimensional neural vector representations). It
# uses a convolution network to bind two Semantic Pointers and a Sum
# network to cojoin two semantic pointers.

# **Note: This model can be simplified if built using the spa (semantic
# pointer architecture) package in Nengo 2.0. This method is shown in the
# spa_structure.py file in the same folder.

# This model has parameters as described in the book, with the ensembles
# having 20 dimensions and 300 neurons each. You will use the
# 'nengo.networks.CircularConvolution' class in Nengo 2.0 to compute the
# convolution (or binding) of two semantic pointers A and B.

# Since the collection of named vectors in a space forms a kind of "vocabulary"
# as described in the book, you will create a vocabulary to build structured
# representations out of it.

# Press the play button to run the simulation.
# The value graphs show the value of individual components of their respective
# ensembles. They show the same information as the "value" graphs in the
# Interactive Plots in Nengo 1.4 GUI as described in the book.

# Sum looks similar to B and A, whereas C looks very dissimilar because it
# is the result of a circular convolution.

import nengo

# Setup the environment
import numpy as np
from nengo.spa import Vocabulary

dim = 20  # Number of dimensions
n_neurons = 300  # Number of neurons in each ensemble

# Creating a vocabulary
rng = np.random.RandomState(0)
vocab = Vocabulary(dimensions=dim, rng=rng)

# Create the network object to which we can add ensembles, connections, etc.
model = nengo.Network(label="Structured Representation")
with model:
    # Input - Get the raw vectors for the pointers using `vocab['A'].v`
    input_A = nengo.Node(output=vocab["A"].v, label="Input A")
    input_B = nengo.Node(output=vocab["B"].v, label="Input B")

    # Ensembles with 300 neurons and 20 dimensions
    # Represents input_A
    ens_A = nengo.Ensemble(n_neurons, dimensions=dim, label="A")
    # Represents input_B
    ens_B = nengo.Ensemble(n_neurons, dimensions=dim, label="B")

    # Represents the convolution of A and B
    ens_C = nengo.Ensemble(n_neurons, dimensions=dim, label="C")
    # Represents the sum of A and B
    ens_sum = nengo.Ensemble(n_neurons, dimensions=dim, label="Sum")

    # Creating the circular convolution network with 70 neurons per dimension
    net_bind = nengo.networks.CircularConvolution(70, dimensions=dim, label="Bind")

    # Connecting the input to ensembles A and B
    nengo.Connection(input_A, ens_A)
    nengo.Connection(input_B, ens_B)

    # Projecting ensembles A and B to the Bind network
    nengo.Connection(ens_A, net_bind.A)
    nengo.Connection(ens_B, net_bind.B)
    nengo.Connection(net_bind.output, ens_C)

    # Projecting ensembles A and B to the Sum ensemble
    nengo.Connection(ens_A, ens_sum)
    nengo.Connection(ens_B, ens_sum)



import nengo
import numpy as np

model = nengo.Network()
with model:
    stimulus = nengo.Node(lambda t: (np.sin(t), np.cos(t)))
    ens = nengo.Ensemble(n_neurons=1000, dimensions=2)
    nengo.Connection(stimulus, ens)

if __name__ == "__main__":
    import nengo_gui

    nengo_gui.GUI(__file__).start()



import argparse
import errno
import logging
import os.path
import threading
import webbrowser

import nengo_gui
import nengo_gui.gui
from nengo_gui.guibackend import GuiServerSettings, ModelContext
from nengo_gui.password import gensalt, hashpw, prompt_pw


def old_main():
    print("'nengo_gui' has been renamed to 'nengo'.")
    print("Please run 'nengo' in the future to avoid this message!\n")
    main()


def main():
    try:
        browser_help = 'browser to use (options: "%s")' % '", "'.join(
            webbrowser._tryorder
        )
    except (AttributeError, TypeError):
        # just in case the undocumented webbrowser._tryorder changes
        browser_help = "browser to use (e.g. chrome, firefox)"

    parser = argparse.ArgumentParser()
    parser.add_argument(
        "-p",
        "--password",
        dest="password",
        metavar="PASS",
        nargs="?",
        default=False,
        const=True,
        type=str,
        help="password for remote access",
    )
    parser.add_argument(
        "-l",
        "--listen",
        nargs=1,
        default=["localhost"],
        type=str,
        help="Network interface to listen on for incoming connections. Set to "
        "'*' to allow connections on all network interfaces.",
    )
    parser.add_argument(
        "--cert", nargs=1, default=[None], type=str, help="SSL certificate file"
    )
    parser.add_argument("--key", nargs=1, default=[None], type=str, help="SSL key file")
    parser.add_argument(
        "--unsecure", action="store_true", help="do not use SSL security"
    )
    parser.add_argument(
        "-P",
        "--port",
        dest="port",
        metavar="PORT",
        type=int,
        help="port to run server on",
    )
    parser.add_argument("filename", nargs="?", type=str, help="initial file to load")
    parser.add_argument("--debug", action="store_true", help="turn on debug logging")
    parser.add_argument(
        "-b",
        "--backend",
        metavar="BACKEND",
        default="nengo",
        type=str,
        help="default backend to use",
    )
    parser.add_argument(
        "--browser",
        dest="browser",
        type=str,
        metavar="BROWSER",
        default=True,
        help=browser_help,
    )
    parser.add_argument("--no-browser", dest="browser", action="store_false")
    parser.add_argument(
        "--auto-shutdown",
        nargs=1,
        type=float,
        help="Time limit before automatic shutdown. Set to 0 to deactivate.",
        default=[2],
    )
    parser.set_defaults(browser=True)
    args = parser.parse_args()

    if args.debug:
        logging.basicConfig(level=logging.DEBUG)
    else:
        logging.basicConfig()

    if args.port is None:
        port = 8080
    else:
        port = args.port

    if args.password:
        if args.password is True:
            password_hash = hashpw(prompt_pw(), gensalt())
        else:
            password_hash = hashpw(args.password, gensalt())
    else:
        password_hash = None

    host = args.listen[0]
    if host == "*":
        host = ""
    server_settings = GuiServerSettings(
        (host, port),
        args.auto_shutdown[0],
        password_hash=password_hash,
        ssl_cert=args.cert[0],
        ssl_key=args.key[0],
    )
    if host != "localhost" and not server_settings.use_ssl and not args.unsecure:
        raise ValueError(
            "Listening on external network interfaces only allowed with SSL."
        )

    try:
        if args.filename is None:
            filename = os.path.join(nengo_gui.__path__[0], "examples", "default.py")
        else:
            filename = args.filename
        page_settings = nengo_gui.page.PageSettings(backend=args.backend)
        s = None
        while s is None:
            try:
                s = nengo_gui.gui.InteractiveGUI(
                    ModelContext(filename=filename),
                    server_settings,
                    page_settings=page_settings,
                )
            except EnvironmentError as err:
                if args.port is None and err.errno == errno.EADDRINUSE:
                    port += 1
                    if port > 0xFFFF:
                        raise
                    server_settings.listen_addr = (host, port)
                else:
                    raise

        s.server.auto_shutdown = args.auto_shutdown[0]

        if args.browser:
            protocol = "https:" if server_settings.use_ssl else "http:"
            host = "localhost"
            port = s.server.server_port
            if args.browser is True:
                wb = webbrowser.get()
            else:
                try:
                    wb = webbrowser.get(args.browser)
                except webbrowser.Error:
                    try:
                        print(
                            "Known browsers: \n  %s" % "\n  ".join(webbrowser._tryorder)
                        )
                    except (AttributeError, TypeError):
                        # just in case the undocumented webbrowser._tryorder
                        #  changes
                        print("Could not determine the list of known browsers.")
                    raise
            t = threading.Thread(
                target=wb.open,
                args=(
                    "%s//%s:%d%s/?token=%s"
                    % (
                        protocol,
                        host,
                        port,
                        s.server.settings.prefix,
                        s.server.gen_one_time_token(),
                    ),
                ),
            )
            t.start()

        s.start()
    finally:
        logging.shutdown()


if __name__ == "__main__":
    main()



from __future__ import print_function

import time

import pytest
from nengo_gui import testing_tools as tt
from selenium.webdriver import ActionChains

test_files = tt.folder_location("examples/basics")


@pytest.mark.parametrize("test_file", test_files)
def test_basic_functionality(driver, test_file):
    # Test page response by clicking the reset button and applying
    # new code to ace-editor
    tt.reset_page(driver)
    time.sleep(0.5)
    tt.update_editor(driver, test_file)
    tt.mouse_scroll(driver, 500)
    ens_elements = driver.find_elements_by_xpath('//*[@class="ens"]')
    assert len(ens_elements) > 0
    side_script = (
        "var right = document.getElementById('rightpane');\n"
        "right.style.width = '200px';\n"
    )
    driver.execute_script(side_script)

    # Creates graph objects by right clicking and selecting
    actions = ActionChains(driver)
    elements = ["node", "ens"]
    for elem in elements:
        node = driver.find_element_by_xpath('//*[@class="' + elem + '"]')
        actions = ActionChains(driver)
        actions.move_to_element(node)
        actions.context_click()
        actions.perform()
        time.sleep(1)
        actions = ActionChains(driver)

        menu = driver.find_element_by_xpath('//*[@class="dropdown-menu"]/li[1]')
        actions.move_to_element(menu)
        actions.click()
        actions.perform()
        time.sleep(0.5)

    graph_elements = driver.find_elements_by_xpath('//*[@class="graph"]')

    assert len(graph_elements) > 0
    tt.start_stop_sim(driver)
    time.sleep(1.5)
    tt.start_stop_sim(driver)
    time.sleep(0.5)

    ticker = driver.find_element_by_xpath('//*[@id="ticks_tr"]/td')
    sim_time = ticker.get_attribute("textContent")

    assert float(sim_time) > 0



import json

from nengo.utils.progress import ProgressBar, timestamp2timedelta
from nengo_gui.compat import escape
from nengo_gui.components.component import Component


class Progress(Component, ProgressBar):
    def __init__(self):
        super(Progress, self).__init__()
        self.progress = None

    def update(self, progress):
        self.progress = progress

    def update_client(self, client):
        if self.progress is not None:
            client.write_text(
                json.dumps(
                    {
                        "name_during": escape(
                            getattr(self.progress, "name_during", "Building")
                        ),
                        "name_after": escape(
                            getattr(self.progress, "name_after", "Build")
                        ),
                        "progress": self.progress.progress,
                        "max_steps": self.progress.max_steps,
                        "elapsed_time": str(
                            timestamp2timedelta(self.progress.elapsed_seconds())
                        ),
                        "eta": str(timestamp2timedelta(self.progress.eta())),
                        "finished": self.progress.finished,
                        "success": self.progress.success,
                    }
                )
            )
            if self.progress.finished:
                self.progress = None

    def javascript(self):
        info = dict(uid=id(self))
        js = self.javascript_config(info)
        return "progress = new Nengo.Progress({});\n".format(js)



# Routed Sequencing with Cleanup all Memory

# The previous model on Routed Sequencing with cleanup memory used a scalar
# ensemble to project only state 'A'. In this model, you will project all the
# states onto an ensemble of neurons as the state cycles through a five element
# sequence.

# This model has parameters as described in the book. It extends the routed
# sequencing model by creating a scalar ensemble 'cleanup' and then projecting
# all the states on to this ensemble using a transformation matrix 'pd_new',
# specified by the vectors in the vocabulary.

# Press the play button to run the simulation.
# The graph in the middle shows the semantic pointer representation of the
# values stored in state and the graph on the bottom-right shows the response of
# the cleanup population. and the graph on the top-right shows the utility
# (similarity) of the current basal ganglia input (i.e., state) with the
# possible vocabulary vectors.

# Since the cleanup operation is similar to a dot product between the state and
# the defined vocabulary vectors, the value of the cleanup population in a
# particular dimension rises only when the value of state (top-right graph)
# corresponds to that particular dimension.

import nengo

# Setup the environment
import numpy as np
from nengo import spa  # import spa related packages
from nengo.spa import Vocabulary

# Number of dimensions for the Semantic Pointers
dim = 16

# Change the seed of this RNG to change the vocabulary
rng = np.random.RandomState(4)
vocab = Vocabulary(dimensions=dim, rng=rng, max_similarity=0.1)

# Make a model object with the SPA network
model = spa.SPA(label="Routed_Sequence with cleanupAll", vocabs=[vocab])

with model:
    # Specify the modules to be used
    model.state = spa.State(dimensions=dim, feedback=1, feedback_synapse=0.01)
    model.vision = spa.State(dimensions=dim)
    # Specify the action mapping
    actions = spa.Actions(
        "dot(vision, START) --> state = vision",
        "dot(state, A) --> state = B",
        "dot(state, B) --> state = C",
        "dot(state, C) --> state = D",
        "dot(state, D) --> state = E",
        "dot(state, E) --> state = A",
    )

    # Creating the BG and thalamus components that confirm to the specified rules
    model.bg = spa.BasalGanglia(actions=actions)
    model.thal = spa.Thalamus(model.bg)

    # Get vocabulary items in order of creation
    vsize = len((model.get_output_vocab("state").keys))
    vocab_items = []
    for index in range(vsize):
        vocab_items.append(model.get_output_vocab("state").keys[index])

    # Creating the transformation matrix (pd) and cleanup SPA State (cleanup)
    pd = []
    for item in vocab_items:
        pd.append([model.get_output_vocab("state")[item].v.tolist()])

    model.cleanup = nengo.Ensemble(n_neurons=300, dimensions=vsize)

    # Function that provides the model with an initial input semantic pointer.
    def start(t):
        if t < 0.4:
            return "0.8*START+D"
        return "0"

    # Input
    model.input = spa.Input(vision=start)

    # Projecting the state on to the cleanup ensemble using a transformation matrix 'pd'
    # Note that the first item in the vocabulary (`START`) is ignored.
    for i in range(1, vsize):
        nengo.Connection(model.state.output, model.cleanup[i], transform=pd[i])



# Representing a Vector

# In nengo, ensembles of neurons represent information. In this model,
# you will represent a two-dimensional vector with a single ensemble of
# LIF (leaky integrate-and-fire) neurons. Though this is a case of
# two-dimensional vector representation, but the ideas naturally
# generalize to any number of dimensions.

# This model has paramenters as described in the book, with the neurons
# in the ensemble having two dimensions. Since the default max rates in
# Nengo 2.0 are (200, 400), so you have to explicitly specify them to be
# (100, 200) to create the same model as in the book. The default values
# of tauRC, tauRef, intercepts, radius and expected noise in Nengo 2.0
# are same as those mentioned in the book.

# Press the play button to run the simulation.
# The cos and sin graphs show the two dimensional input provided to the
# ensemble and the top right graph shows the the decoded estimate of this
# two dimensional input. The graph on bottom right shows the XY-value i.e.,
# the state represented by one dimension of the ensemble vs the state
# represented by the other dimension.

# Grab and move the sliders to manually control the input to the neurons.

import nengo

# Setup the envirnment
import numpy as np
from nengo.dists import Uniform

# Create the network object to which we can add ensembles, connections, etc.
model = nengo.Network(label="2D Representation")

with model:
    # Input Nodes
    stim_sin = nengo.Node(output=np.sin, label="sin")
    stim_cos = nengo.Node(output=np.cos, label="cos")

    # Ensemble with 100 LIF neurons which represents a 2-dimensional signal
    x = nengo.Ensemble(100, dimensions=2, max_rates=Uniform(100, 200))

    # Get the neuron encoders
    encoders = x.encoders.sample(100, 2)

    # Connecnting input to ensemble
    # The indices in ensemble 'x' define which dimension the input will project to
    nengo.Connection(stim_sin, x[0])
    nengo.Connection(stim_cos, x[1])



"""Password hashing functions replicating bcrypt API."""

from __future__ import print_function

import binascii
import hashlib
import os
from getpass import getpass


def gensalt(size=16):
    return binascii.hexlify(os.urandom(size)).decode("ascii")


def hashpw(password, salt, algorithm="sha1"):
    h = hashlib.new(algorithm)
    h.update(password.encode("utf-8"))
    h.update(salt.encode("ascii"))
    return algorithm + ":" + salt + ":" + h.hexdigest()


def checkpw(password, hashed):
    algorithm, salt, _ = hashed.split(":")
    return hashpw(password, salt, algorithm) == hashed


def prompt_pw():
    while True:
        p0 = getpass("Enter password: ")
        p1 = getpass("Enter password: ")
        if p0 == p1:
            return p0
        print("Passwords do not match. Please try again.")



# Sequencing

# This model uses the basal ganglia model to cycle through a sequence of five
# states (i.e., A->B->C->D->E->A->...). The model incorporates a
# working memory component (state), which allows the basal ganglia to update
# that memory based on a set of condition/action mappings.

# The model has parameters as described in the book. In Nengo 1.4 (and the book)
# separate 'Rules' and 'Sequence' classes were created. However, this is not
# needed in Nengo 2.0 since you can directly specify the rules using the
# built-in 'Actions' class in the spa (semantic pointer architecutre) package. This
# class takes a string definition of the action as an input as shown in the code
# where '-->' is used to split the action into condition and effect, otherwise
# it is treated as having no condition and just effect.

# The syntax for creating an input function in Nengo 2.0 is also different from
# that in Nengo 1.4 mentioned in the book. The syntax for Nengo 2.0 which is
# used here is spa.input(module=function). The first parameter 'module'
# refers to name of the module that you want to provide input to and the second
# parameter 'function' refers to the function to execute to generate inupts to
# that module. The functions should always return strings, which will then be
# parsed by the relevant vocabulary.

# In Nengo 1.4, a memory element for representing the state was created by using
# Buffer() as described in the book. However, in Nengo 2.0, we use
# State() with feedback parameter set to 1 for creating a memory module capable
# of storing a vector over time.

# Press the play button in the visualizer to run the simulation.
# The graph on the top-left shows the semantic pointer representation of the
# values stored in the state ensemble. The graph on the bottom-right shows the
# current transition or the action being executed, and the graph on the
# top-right shows the utility (similarity) of the current basal ganglia input
# (i.e., state) with the possible vocabulary vectors.

# The book describes that the results of the model can be seen through the
# visualizer in Nengo 1.4 GUI which has a "Utility" box and the "Rules" box.
# Note that the bottom-right graph shows the same information as seen in the
# "Rules" box and top-right graph shows the same information as seen in the
# "Utility" box.

# Setup the environment
import nengo
from nengo import spa  # import spa related packages

# Number of dimensions for the Semantic Pointers
dim = 16

# Create the spa.SPA network to which we can add SPA objects
model = spa.SPA(label="Sequence")
with model:
    # Creating a working memory/cortical element
    model.state = spa.State(dimensions=dim, feedback=1, feedback_synapse=0.01)

    # Specifying the action mappings (rules) for BG and Thal
    actions = spa.Actions(
        "dot(state, A) --> state = B",
        "dot(state, B) --> state = C",
        "dot(state, C) --> state = D",
        "dot(state, D) --> state = E",
        "dot(state, E) --> state = A",
    )

    # Creating the BG and thalamus components that confirm to the specified rules
    model.bg = spa.BasalGanglia(actions=actions)
    model.thal = spa.Thalamus(model.bg)

    # Function that provides the model with an initial input semantic pointer.
    def start(t):
        if t < 0.1:  # Duration of the initial input = 0.1
            return "D"
        return "0"

    # Input
    model.input = spa.Input(state=start)



import time

from nengo_gui import testing_tools as tt


def set_cloud_value(driver, spa_nets):
    for name in spa_nets:
        prompt_script = """
var items = Nengo.Component.components;
items.forEach(function(pointer) {
    if (pointer.label.innerHTML === '%s' &&
            pointer.constructor === Nengo.Pointer) {
        pointer.set_value();
    }
});
"""
        time.sleep(0.6)
        driver.execute_script(prompt_script % name)
        time.sleep(1.0)
        field = driver.find_element_by_xpath("//*[@id='singleInput']")
        time.sleep(0.6)
        field.send_keys(spa_nets[name])
        accept = driver.find_element_by_xpath("//*[@id='OK']")
        time.sleep(0.3)
        accept.click()


def test_spa(driver):
    """Tests the functionality of SPA simulations"""

    test_file = """
import nengo
import nengo.spa as spa

D = 2  # the dimensionality of the vectors

model = spa.SPA()
with model:
    model.color = spa.State(D)
    model.shape = spa.State(D)
    model.memory = spa.State(D, feedback=1)
    model.query = spa.State(D)
    model.answer = spa.State(D)

    actions = spa.Actions(
        "memory = color * shape",
        "answer = memory * ~query",
        )

    model.cortical = spa.Cortical(actions)
"""
    tt.reset_page(driver)
    time.sleep(0.2)
    tt.update_editor(driver, test_file)
    tt.mouse_scroll(driver, 200)

    # Generates semantic pointer clouds for each network
    driver.execute_script(
        """
var a = Nengo.netgraph.svg_objects;
for (model in a) {
    if (a[model].sp_targets.length > 0) {
        a[model].create_graph('Pointer', a[model].sp_targets[0]);
        a[model].create_graph('SpaSimilarity', a[model].sp_targets[0]);
    }
}
"""
    )
    time.sleep(1)
    # Ensures the simulation has started

    hang_time = 100  # alloted time until test fails
    time_script = "var time = $('#ticks_tr');\n" "return time.find('td').text();\n"
    tt.start_stop_sim(driver)
    time_start = time.time()
    while time.time() - time_start < hang_time:
        sim_time = driver.execute_script(time_script)
        if float(sim_time) > 0:
            break
        time.sleep(1)
    else:
        assert False, "Did not compile in %f seconds" % hang_time

    # Sets the semantic pointers appropriately
    spa_values = {"shape": "CIRCLE", "color": "BLUE", "query": "CIRCLE"}
    set_cloud_value(driver, spa_values)
    time.sleep(10)
    result = driver.execute_script(
        """
var objects = Nengo.Component.components;
var answer = objects.filter(function(item) {
    return item.label.innerHTML == "answer";
})[0];
var answer_data = answer.data_store.data[0];
var result = answer_data.pop();
return result;
"""
    )

    data_script = """
return Nengo.Component.components.filter(function(item) {
    return (item.constructor === Nengo.SpaSimilarity);
})[0].data_store.data;
"""
    plot_data = driver.execute_script(data_script)

    # Checks that the 'answer' is BLUE
    assert any("BLUE" in x for x in result)

    # Checks the dimensionality of the spa similarity plot
    assert len(plot_data) == 2 and len(plot_data[0]) > 1

    set_cloud_value(driver, {"color": "ORANGE"})

    time.sleep(10)

    plot_data = driver.execute_script(data_script)

    # Checks that the data store grows when input changes
    assert len(plot_data) == 3 and len(plot_data[0]) > 1



# Question Answering

# This model shows a simple form of question answering. The goal of
# this model is to build a network that will output answers to questions
# based on supplied statements as described in the book. You will bind
# two features (color and shape) by circular convolution. Then you will provide
# a cue to the model to determine either one of the features by deconvolution.

# **Note: Simplified method of building the model using the spa
# (semantic pointer architecture) package in Nengo 2.0 is shown
# in the spa_question.py file in the same folder.

# This model has parameters as described in the book, with the ensembles having
# 300 neurons in 20 dimensions and the bind and unbind networks having 70
# neurons per dimension. The model relies on SPA (Semantic Pointer Architecture)
# methods for charachterizing representation, transformation and control.

# Depending on your computer, this model may run slowly which can be avoided by
# several ways as discussed in the book. In order to run the model in direct
# mode (explained in the book), the parameter 'neuron_type' should be set to
# nengo.Direct() while creating the ensembles.

# As mentioned in the book, the color input will alternate every 0.5 seconds
# between RED and BLUE. In the same way the shape input alternates between
# CIRCLE and SQUARE. Thus, the model will bind alternatingly RED * CIRCLE and
# BLUE * SQUARE for 0.5 seconds each. In parallel you will ask a question from
# the model by providing a cue which will be used for deconvolving bound
# semantic pointers to determine an answer. For example, when the cue is CIRCLE
# the model will respond with RED. The cue will cycle through CIRCLE, RED,
# SQUARE, and BLUE within one second.

# Press the play button to run the simulation.
# The graph labelled "output" shows that the output is most similar to the semantic
# pointer bound to the current cue. For example, when RED and CIRCLE are being
# bound and the cue is CIRCLE, the output is most similar to RED.  This is much
# easier to see in the spa_question.py version.

import nengo

# Setup the environment
import numpy as np
from nengo.spa import Vocabulary

dim = 20  # Number of dimensions
n_neurons = 300  # Number of neurons in population
n_conv = 70  # Number of neurons per dimension in bind/unbind populations

rng = np.random.RandomState(0)
vocab = Vocabulary(dimensions=dim, rng=rng, max_similarity=0.1)
model = nengo.Network(label="Question Answering", seed=7)

with model:
    # Ensembles
    ens_A = nengo.Ensemble(n_neurons=n_neurons, dimensions=dim, label="A")
    ens_B = nengo.Ensemble(n_neurons=n_neurons, dimensions=dim, label="B")
    ens_C = nengo.Ensemble(n_neurons=n_neurons, dimensions=dim, label="C")
    ens_D = nengo.Ensemble(n_neurons=n_neurons, dimensions=dim, label="D")
    ens_E = nengo.Ensemble(n_neurons=n_neurons, dimensions=dim, label="E")

    # Creating the bind network
    net_bind = nengo.networks.CircularConvolution(n_neurons=n_conv, dimensions=dim)
    nengo.Connection(ens_A, net_bind.A)
    nengo.Connection(ens_B, net_bind.B)
    nengo.Connection(net_bind.output, ens_D)

    # Creating the Unbind network
    net_unbind = nengo.networks.CircularConvolution(
        n_neurons=n_conv, dimensions=dim, invert_a=True
    )
    nengo.Connection(ens_C, net_unbind.A)
    nengo.Connection(ens_D, net_unbind.B)
    nengo.Connection(net_unbind.output, ens_E)

    # Getting semantic pointer values
    CIRCLE = vocab.parse("CIRCLE").v
    BLUE = vocab.parse("BLUE").v
    RED = vocab.parse("RED").v
    SQUARE = vocab.parse("SQUARE").v
    ZERO = [0] * dim

    # Function for providing color input
    def color_input(t):
        if (t // 0.5) % 2 == 0:
            return RED
        return BLUE

    # Function for providing shape input
    def shape_input(t):
        if (t // 0.5) % 2 == 0:
            return CIRCLE
        return SQUARE

    # Function for providing the cue
    def cue_input(t):
        sequence = [ZERO, CIRCLE, RED, ZERO, SQUARE, BLUE]
        idx = int((t // (1.0 / len(sequence))) % len(sequence))
        return sequence[idx]

    # Defining inputs
    input_A = nengo.Node(output=color_input, size_out=dim, label="Input A")
    input_B = nengo.Node(output=shape_input, size_out=dim, label="Input B")
    input_C = nengo.Node(output=cue_input, size_out=dim, label="Input C")

    # Connecting input to ensembles
    nengo.Connection(input_A, ens_A)
    nengo.Connection(input_B, ens_B)
    nengo.Connection(input_C, ens_C)



import inspect

import nengo
import nengo_gui.components


def make_param(name, default):
    try:
        # the most recent way of making Parameter objects
        p = nengo.params.Parameter(name=name, default=default)
    except TypeError:
        # this was for older releases of nengo (v2.0.3 and earlier)
        p = nengo.params.Parameter(default=default)
    return p


class Config(nengo.Config):
    def __init__(self):
        super(Config, self).__init__()
        for cls in [nengo.Ensemble, nengo.Node]:
            self.configures(cls)
            self[cls].set_param("pos", make_param(name="pos", default=None))
            self[cls].set_param("size", make_param(name="size", default=None))
        self.configures(nengo.Network)
        self[nengo.Network].set_param("pos", make_param(name="pos", default=None))
        self[nengo.Network].set_param("size", make_param(name="size", default=None))
        self[nengo.Network].set_param(
            "expanded", make_param(name="expanded", default=False)
        )
        self[nengo.Network].set_param(
            "has_layout", make_param(name="has_layout", default=False)
        )

        for clsname, cls in inspect.getmembers(nengo_gui.components):
            if inspect.isclass(cls):
                if issubclass(cls, nengo_gui.components.component.Component):
                    if cls != nengo_gui.components.component.Component:
                        self.configures(cls)
                        for k, v in cls.config_defaults.items():
                            p = make_param(name=k, default=v)
                            self[cls].set_param(k, p)

    def dumps(self, uids):
        lines = []
        for obj, uid in sorted(uids.items(), key=lambda x: x[1]):

            if isinstance(obj, (nengo.Ensemble, nengo.Node, nengo.Network)):
                if self[obj].pos is not None:
                    lines.append("_viz_config[%s].pos=%s" % (uid, self[obj].pos))
                if self[obj].size is not None:
                    lines.append("_viz_config[%s].size=%s" % (uid, self[obj].size))
                if isinstance(obj, nengo.Network):
                    lines.append(
                        "_viz_config[%s].expanded=%s" % (uid, self[obj].expanded)
                    )
                    lines.append(
                        "_viz_config[%s].has_layout=%s" % (uid, self[obj].has_layout)
                    )

            elif isinstance(obj, nengo_gui.components.editor.Editor):
                pass  # Do not persist editor

            elif isinstance(obj, nengo_gui.components.component.Component):
                lines.append("%s = %s" % (uid, obj.code_python(uids)))
                for k in obj.config_defaults.keys():
                    v = getattr(self[obj], k)
                    val = repr(v)

                    try:
                        recovered_v = eval(val, {})
                    except:
                        raise ValueError(
                            "Cannot save %s to config. Only "
                            "values that can be successfully "
                            "evaluated are allowed." % (val)
                        )

                    if recovered_v != v:
                        raise ValueError(
                            "Cannot save %s to config, recovery "
                            "failed. Only "
                            "values that can be recovered after "
                            "being entered into the config file "
                            "can be saved." % (val)
                        )

                    lines.append("_viz_config[%s].%s = %s" % (uid, k, val))

        return "\n".join(lines)



from __future__ import division

import struct

import nengo
import numpy as np
from nengo_gui.components.component import Component


class Voltage(Component):
    """Represents neuron voltage over time."""

    config_defaults = dict(
        max_value=5.0,
        min_value=0.0,
        show_legend=False,
        legend_labels=[],
        **Component.config_defaults,
    )

    def __init__(self, obj, n_neurons=5):
        super(Voltage, self).__init__()
        self.obj = obj.neurons
        self.data = []
        self.max_neurons = int(self.obj.size_out)
        self.n_neurons = min(n_neurons, self.max_neurons)
        self.struct = struct.Struct("<%df" % (1 + self.n_neurons))

    def attach(self, page, config, uid):
        super(Voltage, self).attach(page, config, uid)
        self.label = page.get_label(self.obj.ensemble)

    def add_nengo_objects(self, page):
        with page.model:
            self.probe = nengo.Probe(self.obj[: self.n_neurons], "voltage")

    def remove_nengo_objects(self, page):
        page.model.probes.remove(self.probe)

    def format_data(self, t, x):
        data = self.struct.pack(t, *x[: self.n_neurons])
        self.data.append(data)

    def update_client(self, client):
        sim = self.page.sim
        if sim is None:
            return

        # TODO: this is hack to delete probe data in Nengo 2.0.1, since we
        # can't limit the size of probes. Fix this up with Nengo 2.1.
        data = sim.data.raw[self.probe][:]
        del sim.data.raw[self.probe][:]  # clear the data
        trange = sim.trange()[-len(data) :]

        for t, datum in zip(trange, data):
            datum = datum + np.arange(self.n_neurons)
            packet = self.struct.pack(t, *datum)
            client.write_binary(packet)

    def javascript(self):
        info = dict(uid=id(self), label=self.label, n_lines=self.n_neurons, synapse=0)
        json = self.javascript_config(info)
        return "new Nengo.Value(main, sim, %s);" % json

    def code_python_args(self, uids):
        return [uids[self.obj.ensemble]]



# 2D Decision Integrator

# This is a model of perceptual decision making using a two dimensional
# integrator. As mentioned in the book, the goal is to construct a simple model
# of perceptual decision making without being concerned with establishing a
# detailed connection to neurobiology (we have done this elsewhere).

# Rather than having two different integrators for each dimension, you will
# build the model using a single two dimensional integrator. This integrator
# can be used irrespective of the task demands since it effectively integrates
# in every direction simultaneously. This is neurally more efficient due to the
# reasons explained in the book.

# The model has four ensembles: MT representing the motion area, LIP
# representing the lateral intraparietal area, input and output of the 2D
# integrator. The parameters used in the model are as described in the book.
# The 2D integrator resides in LIP. As discussed in the book an integrator
# requires two connections: here, the input from MT to LIP and the feedback
# connection from LIP to LIP.

# Here, you will provide a an input of (-0.5, 0.5) to the model spanning over a
# period of 6 seconds to observe the model behaviour. In order to inject noise
# while the simulation runs, you can use the 'noise' parameter when creating
# ensembles as shown. The reason for injecting noise is explained in the book.

# Press the play button to run the simulation.
# The output plot on the bottom-right shows the output of the 2D decision
# integrator which is represented by a single two dimensional output ensemble.
# You can see that as MT encodes the input over time, LIP slowly moves towards
# the same direction as it acuumulates evidence that there is sustained motion
# in that direction.

# Thus MT moves LIP in the right direction and once past a certain threshold,
# the output neurons start firing. To visualize this:
# 1) Select 'spikes' from the right-click menu of the output ensemble. This will
#    display a spike plot.
# 2) Run the simulation and then slide the blue box in the simulation control
#    bar backwards.
# 3) You will see that the spikes become stronger once past a certain threshold
#    (i.e., when LIP starts following MT)

# Setup the environment
import nengo
from nengo.dists import Uniform
from nengo.processes import WhiteNoise

# Create the network object to which we can add ensembles, connections, etc.
model = nengo.Network(label="2D Decision Integrator", seed=11)

with model:
    # Inputs
    input1 = nengo.Node(-0.5, label="Input 1")
    input2 = nengo.Node(0.5, label="Input 2")

    # Ensembles
    ens_inp = nengo.Ensemble(100, dimensions=2, label="Input")
    MT = nengo.Ensemble(100, dimensions=2, noise=WhiteNoise(dist=Uniform(-0.3, 0.3)))
    LIP = nengo.Ensemble(200, dimensions=2, noise=WhiteNoise(dist=Uniform(-0.3, 0.3)))
    ens_out = nengo.Ensemble(
        100,
        dimensions=2,
        intercepts=Uniform(0.3, 1),
        noise=WhiteNoise(dist=Uniform(-0.3, 0.3)),
        label="Output",
    )

    weight = 0.1
    # Connecting the input signal to the input ensemble
    nengo.Connection(input1, ens_inp[0], synapse=0.01)
    nengo.Connection(input2, ens_inp[1], synapse=0.01)

    # Providing input to MT ensemble
    nengo.Connection(ens_inp, MT, synapse=0.01)

    # Connecting MT ensemble to LIP ensemble
    nengo.Connection(MT, LIP, transform=weight, synapse=0.1)

    # Connecting LIP ensemble to itself
    nengo.Connection(LIP, LIP, synapse=0.1)

    # Connecting LIP population to output
    nengo.Connection(LIP, ens_out, synapse=0.01)



from __future__ import print_function

import inspect
import os
import time

import nengo_gui


def update_editor(driver, nengoCode):

    """Inserts a string which represents code into ace editor

    Example:
    driver = webdriver.firefox()
    driver.get(localhost:8080/)
    code_string = "print hello world"
    update_editor(driver, code_string)
    Ace editor will update with "print hello world"
    """

    nengoCode = nengoCode.replace("\n", "\\n").replace("\r", "\\r")
    js = "var editor = ace.edit('editor');editor.setValue('" + nengoCode + "');"
    driver.execute_script(js)
    time.sleep(1)


def reset_page(driver):

    """Resets the Nengo gui page

    Example:
    driver = webdriver.firefox()
    driver.get(localhost:8080/)
    reset_page(driver)
    The page then resets
    """
    driver.execute_script("toolbar.reset_model_layout();")
    time.sleep(0.3)


def start_stop_sim(driver):
    """Clicks the start simulation start button"""
    play_button = driver.find_element_by_xpath('//*[@id="pause_button"]')
    play_button.click()


def folder_location(var_path, indiv_file=None):

    """Returns a list of the raw text from a python file in var_path

    Example:
    File Structure:
    --nengo_gui/
      --nengo_gui/
             --stuff/
                  --tests/
                      bar.py
                      foo.py

    foo.py:
      print "hello world"
    bar.py:
      print "goodbye world"

    Code:
    files_raw = folder_location('stuff/tests')
    Returns list of ['print "hello world"','print "goodbye world"']
    """

    folder = os.path.dirname(inspect.getfile(nengo_gui))

    test_folder = os.path.join(folder, var_path)

    test_files = os.listdir(test_folder)
    test_files = filter((lambda x: ((x.count(".") == 1) and (".py" in x))), test_files)
    if indiv_file is not None:
        test_files = filter((lambda x: (x == indiv_file)), test_files)
    test_files = map((lambda file_: os.path.join(test_folder, file_)), test_files)
    test_files = map((lambda file_: open(file_, "r").read()), test_files)
    test_files = map((lambda raw_file: raw_file.replace("'", r"\'")), test_files)
    return test_files


def mouse_scroll(driver, scroll_y):

    """scrolls by scroll_y in the netgraph div"""

    element = driver.find_element_by_id("netgraph")
    mouse_x = (element.location["x"] + element.size["width"]) / 2.0
    mouse_y = (element.location["y"] + element.size["height"]) / 2.0
    script = """var netg = document.getElementById("netgraph");
                evt = document.createEvent("Event");
                evt.initEvent("wheel", true, true);
                evt.deltaY = %s;
                evt.clientX = %s;
                evt.clientY = %s ;
                netg.dispatchEvent(evt);"""
    driver.execute_script(script % (scroll_y, mouse_x, mouse_y))



#!/usr/bin/env python

# This code is part of Grandalf
# Copyright (C) 2010-2012 Axel Tillequin (bdcht3@gmail.com)
# published under the GPLv2 license or EPLv1 license

from bisect import bisect
from functools import cmp_to_key
from sys import getrecursionlimit, setrecursionlimit

#  Layouts are classes that provide graph drawing algorithms.
#
#  These classes all take a graph_core argument. The graph_core
#  topology will never be permanently modified by the drawing algorithm:
#  e.g. "dummy" node insertion, edge reversal for making the graph
#  acyclic and so on, are all kept inside the layout object.
#
from numpy import array, matrix

from .utils import median_wh, rand_ortho1

try:
    xrange
except NameError:
    xrange = range

try:
    from itertools import izip
except:
    izip = zip

#  the VertexViewer class is responsible of providing
#  graphical attributes associated with a Vertex.
#  Instance of VertexViewer are generally created from
#  Vertex.data and then embedded in the 'view' field.
#  It is used by the layout algorithms the get vertex dimensions.
class VertexViewer(object):
    def __init__(self, w=2, h=2, data=None):
        self.w = w
        self.h = h

    def __str__(self, *args, **kwargs):
        if hasattr(self, "xy"):
            return "VertexViewer (xy: %s) w: %s h: %s" % (self.xy, self.w, self.h)

        return "VertexViewer (xy: None) w: %s h: %s" % (self.w, self.h)


#  SUGIYAMA LAYOUT
# ------------------------------------------------------------------------------
#  The Sugiyama layout is the traditional "layered" graph layout
#  named 'dot' in graphviz.
#  This layout is quite efficient but heavily relies on drawing
#  heuristics. Adaptive drawing is limited to
#  extending the leaves only, but since the algorithm is quite fast
#  redrawing the entire graph (up to about a thousand nodes) gives
#  usually better results in less than a second.


class _sugiyama_vertex_attr(object):
    def __init__(self, r=None, d=0):
        self.rank = r
        self.dummy = d
        self.pos = None
        self.x = 0
        self.bar = None

    def __str__(self):
        s = "(%3d,%3d) x=%s" % (self.rank, self.pos, str(self.x))
        if self.dummy:
            s = "[d] %s" % s
        return s


class DummyVertex(_sugiyama_vertex_attr):
    def __init__(self, r=None, viewclass=VertexViewer):
        self.view = viewclass()
        self.ctrl = None
        self.constrainer = False
        self.controlled = False
        _sugiyama_vertex_attr.__init__(self, r, d=1)

    def N(self, dir):
        assert dir == +1 or dir == -1
        return self.ctrl.get(self.rank + dir, [])

    def inner(self, dir):
        assert dir == +1 or dir == -1
        try:
            return self.ctrl[self.rank + dir][-1].dummy == 1
        except KeyError:
            return False
        except AttributeError:
            return False

    def __str__(self):
        s = "(%3d,%3d) x=%s" % (self.rank, self.pos, str(self.x))
        if self.dummy:
            s = "[d] %s" % s
        if self.constrainer:
            s = "[c] %s" % s
        return s


# ------------------------------------------------------------------------------
# Layer is where Sugiyama layout organises nodes in hierarchical lists.
# The placement of nodes is done by the Sugiyama class, but it highly relies on
# the 'ordering' of nodes in each layer to reduce crossings.
# This ordering depends on the neighbors found in the upper or lower layers.
# WARNING: methods HIGHLY depend on layout.dirv state.
# TODO: see if grx dict can be stored inside Layers or needs to stay in layout.
# ------------------------------------------------------------------------------
class Layer(list):
    __r = None
    layout = None
    upper = None
    lower = None
    __x = 1.0
    ccount = None

    def __str__(self):
        s = "<Layer %d" % self.__r
        s += ", len=%d" % len(self)
        xc = self.ccount or "?"
        s += ", crossings=%s>" % xc
        return s

    def setup(self, layout):
        self.layout = layout
        r = layout.layers.index(self)
        self.__r = r
        if len(self) > 1:
            self.__x = 1.0 / (len(self) - 1)
        for i, v in enumerate(self):
            assert layout.grx[v].rank == r
            layout.grx[v].pos = i
            layout.grx[v].bar = i * self.__x
        if r > 0:
            self.upper = layout.layers[r - 1]
        if r < len(layout.layers) - 1:
            self.lower = layout.layers[r + 1]

    def nextlayer(self):
        return self.lower if self.layout.dirv == -1 else self.upper

    def prevlayer(self):
        return self.lower if self.layout.dirv == +1 else self.upper

    def order(self):
        sug = self.layout
        sug._edge_inverter()
        mvmt = []
        c = self._cc()
        if c > 0:
            for v in self:
                if sug.grx[v].dummy and v.constrainer:
                    mvmt.append(v)
                    continue
                bar = self._meanvalueattr(v, sug.order_attr)
                sug.grx[v].bar = bar
            while len(mvmt) > 0:
                v = mvmt.pop()
                v.bar = sug.grx[v.ctrl[self.__r][0]].bar
            # now resort layers l according to bar value:
            self.sort(
                key=cmp_to_key(
                    lambda x, y: (
                        (sug.grx[x].bar > sug.grx[y].bar)
                        - (sug.grx[x].bar < sug.grx[y].bar)
                    )
                )
            )
            # assign new position in layer l:
            for i, v in enumerate(self):
                if sug.grx[v].pos != i:
                    mvmt.append(v)
                sug.grx[v].pos = i
                # sug.grx[v].bar = i*self.__x
            # try count resulting crossings:
            c = self._ordering_reduce_crossings()
        self.layout._edge_inverter()
        self.ccount = c
        return mvmt

    # find new position of vertex v according to adjacency in prevlayer.
    # position is given by the mean value of adjacent positions.
    # experiments show that meanvalue heuristic performs better than median.
    def _meanvalueattr(self, v, att="bar"):
        sug = self.layout
        if sug.grx[v].dummy and v.constrainer:
            return v.ctrl[self.__r][0].bar
        if not self.prevlayer():
            return getattr(sug.grx[v], att)
        pos = [getattr(sug.grx[x], att) for x in self._neighbors(v)]
        if len(pos) == 0:
            return getattr(sug.grx[v], att)
        return float(sum(pos)) / len(pos)

    # find new position of vertex v according to adjacency in layer l+dir.
    # position is given by the median value of adjacent positions.
    # median heuristic is proven to achieve at most 3 times the minimum
    # of crossings (while barycenter achieve in theory the order of |V|)
    def _medianindex(self, v):
        assert self.prevlayer() != None
        N = self._neighbors(v)
        g = self.layout.grx
        if g[v].dummy and v.controlled:
            for x in N:
                if g[x].dummy and x.constrainer:
                    return [g[x].pos]
        pos = [g[x].pos for x in N]
        lp = len(pos)
        if lp == 0:
            return []
        pos.sort()
        pos = pos[:: self.layout.dirh]
        i, j = divmod(lp - 1, 2)
        return [pos[i]] if j == 0 else [pos[i], pos[i + j]]

    # neighbors refer to upper/lower adjacent nodes.
    # remember that v.N() provides neighbors of v within the graph, while
    # this method provides the Vertex and DummyVertex adjacent to v in the
    # upper or lower layer (depending on layout.dirv state).
    def _neighbors(self, v):
        assert self.layout.dag
        dirv = self.layout.dirv
        grxv = self.layout.grx[v]
        try:  # (cache)
            return grxv.nvs[dirv]
        except AttributeError:
            grxv.nvs = {-1: v.N(-1), +1: v.N(+1)}
            if grxv.dummy:
                return grxv.nvs[dirv]
            # v is real, v.N are graph neigbors but we need layers neighbors
            for d in (-1, +1):
                tr = grxv.rank + d
                for i, x in enumerate(v.N(d)):
                    if self.layout.grx[x].rank == tr:
                        continue
                    e = v.e_with(x)
                    dum = self.layout.ctrls[e][tr][-1]
                    grxv.nvs[d][i] = dum
            return grxv.nvs[dirv]

    # counts (inefficently but at least accurately) the number of
    # crossing edges between layer l and l+dirv.
    # P[i][j] counts the number of crossings from j-th edge of vertex i.
    # The total count of crossings is the sum of flattened P:
    # x = sum(sum(P,[]))
    def _crossings(self):
        g = self.layout.grx
        P = []
        for v in self:
            P.append([g[x].pos for x in self._neighbors(v)])
        for i, p in enumerate(P):
            candidates = sum(P[i + 1 :], [])
            for j, e in enumerate(p):
                p[j] = len(list(filter((lambda nx: nx < e), candidates)))
            del candidates
        return P

    # implementation of the efficient bilayer cross counting by insert-sort
    # (see Barth & Mutzel paper "Simple and Efficient Bilayer Cross Counting")
    def _cc(self):
        g = self.layout.grx
        P = []
        for v in self:
            P.extend(sorted([g[x].pos for x in self._neighbors(v)]))
        # count inversions in P:
        s = []
        count = 0
        for i, p in enumerate(P):
            j = bisect(s, p)
            if j < i:
                count += i - j
            s.insert(j, p)
        return count

    def _ordering_reduce_crossings(self):
        assert self.layout.dag
        g = self.layout.grx
        N = len(self)
        X = 0
        for i, j in izip(xrange(N - 1), xrange(1, N)):
            vi = self[i]
            vj = self[j]
            ni = [g[v].pos for v in self._neighbors(vi)]
            Xij = Xji = 0
            for nj in [g[v].pos for v in self._neighbors(vj)]:
                x = len(list(filter((lambda nx: nx > nj), ni)))
                Xij += x
                Xji += len(ni) - x
            if Xji < Xij:
                g[vi].pos, g[vj].pos = g[vj].pos, g[vi].pos
                self[i] = vj
                self[j] = vi
                X += Xji
            else:
                X += Xij
        return X


# ------------------------------------------------------------------------------
#  The Sugiyama Layout Class takes as input a core_graph object and implements
#  an efficient drawing algorithm based on nodes dimensions provided through
#  a user-defined 'view' property in each vertex (see README.txt).
# ------------------------------------------------------------------------------
class SugiyamaLayout(object):
    def __init__(self, g):
        # drawing parameters:
        self.dirvh = 0
        self.order_iter = 8
        self.order_attr = "pos"
        self.xspace = 20
        self.yspace = 20
        self.dw = 10
        self.dh = 10
        # For layered graphs, vertices and edges need to have some additional
        # attributes that make sense only for this kind of layout:
        # update graph struct:
        self.g = g
        self.layers = []
        self.grx = {}
        self.ctrls = {}
        self.dag = False
        for v in self.g.V():
            assert hasattr(v, "view")
            self.grx[v] = _sugiyama_vertex_attr()
        self.dw, self.dh = median_wh([v.view for v in self.g.V()])
        self.dw = 8

    # initialize the layout engine based on required
    #  -list of edges for making the graph_core acyclic
    #  -list of root nodes.
    def init_all(self, roots=None, inverted_edges=None, cons=False, optimize=False):
        # For layered sugiyama algorithm, the input graph must be acyclic,
        # so we must provide a list of root nodes and a list of inverted edges.
        if roots == None:
            roots = list(filter(lambda x: len(list(x.e_in())) == 0, self.g.sV))
        if inverted_edges == None:
            L = self.g.get_scs_with_feedback(roots)
            inverted_edges = list(filter(lambda x: x.feedback, self.g.sE))
        self.alt_e = inverted_edges
        # assign rank to all vertices:
        self.rank_all(roots, optimize)
        # add dummy vertex/edge for 'long' edges:
        self.ctrls["cons"] = cons  # use "constrained edges" ?
        for e in self.g.E():
            self.setdummies(e, cons)
        # precompute some layers values:
        for l in self.layers:
            l.setup(self)

    # compute every node coordinates after converging to optimal ordering by N
    # rounds, and finally perform the edge routing.
    def draw(self, N=1.5):
        while N > 0.5:
            for (l, mvmt) in self.ordering_step():
                pass
            N = N - 1
        if N > 0:
            for (l, mvmt) in self.ordering_step(oneway=True):
                pass
        self.setxy()
        self.draw_edges()

    def _edge_inverter(self):
        for e in self.alt_e:
            x, y = e.v
            e.v = (y, x)
        self.dag = not self.dag

    # internal state for alignment policy:
    # dirvh=0 -> dirh=+1, dirv=-1: leftmost upper
    # dirvh=1 -> dirh=-1, dirv=-1: rightmost upper
    # dirvh=2 -> dirh=+1, dirv=+1: leftmost lower
    # dirvh=3 -> dirh=-1, dirv=+1: rightmost lower
    @property
    def dirvh(self):
        return self.__dirvh

    @property
    def dirv(self):
        return self.__dirv

    @property
    def dirh(self):
        return self.__dirh

    @dirvh.setter
    def dirvh(self, dirvh):
        assert dirvh in range(4)
        self.__dirvh = dirvh
        self.__dirh, self.__dirv = {0: (1, -1), 1: (-1, -1), 2: (1, 1), 3: (-1, 1)}[
            dirvh
        ]

    @dirv.setter
    def dirv(self, dirv):
        assert dirv in (-1, +1)
        dirvh = (dirv + 1) + (1 - self.__dirh) / 2
        self.dirvh = dirvh

    @dirh.setter
    def dirh(self, dirh):
        assert dirh in (-1, +1)
        dirvh = (self.__dirv + 1) + (1 - dirh) / 2
        self.dirvh = dirvh

    # rank all vertices.
    # if list l is None, find initial rankable vertices (roots),
    # otherwise update ranking from these vertices.
    # The initial rank is based on precedence relationships,
    # optimal ranking may be derived from network flow (simplex).
    def rank_all(self, roots, optimize=False):
        self._edge_inverter()
        r = list(
            filter(lambda x: len(list(x.e_in())) == 0 and x not in roots, self.g.sV)
        )
        self._rank_init(list(roots) + list(r))
        if optimize:
            self._rank_optimize()
        self._edge_inverter()

    def _rank_init(self, unranked):
        assert self.dag
        scan = {}
        # set rank of unranked based on its in-edges vertices ranks:
        while len(unranked) > 0:
            l = []
            for v in unranked:
                self.setrank(v)
                # mark out-edges has scan-able:
                for e in v.e_out():
                    scan[e] = True
                # check if out-vertices are rank-able:
                for x in v.N(+1):
                    if not (False in [scan.get(e, False) for e in x.e_in()]):
                        if x not in l:
                            l.append(x)
            unranked = l

    # TODO: Network flow solver minimizing total edge length
    # Also interesting: http://jgaa.info/accepted/2005/EiglspergerSiebenhallerKaufmann2005.9.3.pdf
    def _rank_optimize(self):
        assert self.dag
        for l in reversed(self.layers):
            for v in l:
                gv = self.grx[v]
                for x in v.N(-1):
                    if all((self.grx[y].rank >= gv.rank for y in x.N(+1))):
                        gx = self.grx[x]
                        self.layers[gx.rank].remove(x)
                        gx.rank = gv.rank - 1
                        self.layers[gv.rank - 1].append(x)

    def setrank(self, v):
        assert self.dag
        r = max([self.grx[x].rank for x in v.N(-1)] + [-1]) + 1
        self.grx[v].rank = r
        # add it to its layer:
        try:
            self.layers[r].append(v)
        except IndexError:
            assert r == len(self.layers)
            self.layers.append(Layer([v]))

    def dummyctrl(self, r, ctrl):
        dv = DummyVertex(r)
        dv.view.w, dv.view.h = self.dw, self.dh
        self.grx[dv] = dv
        dv.ctrl = ctrl
        try:
            ctrl[r].append(dv)
        except KeyError:
            ctrl[r] = [dv]
        self.layers[r].append(dv)
        return dv

    def setdummies(self, e, with_constraint=True):
        v0, v1 = e.v
        r0, r1 = self.grx[v0].rank, self.grx[v1].rank
        if r0 > r1:
            assert e in self.alt_e
            v0, v1 = v1, v0
            r0, r1 = r1, r0
        elif r0 == r1:
            raise ValueError("bad ranking")
        spanover = xrange(r0 + 1, r1)
        if (r1 - r0) > 1:
            # "dummy vertices" are stored in the edge ctrl dict,
            # keyed by their rank in layers.
            ctrl = self.ctrls[e] = {}
            ctrl[r0] = [v0]
            ctrl[r1] = [v1]
            for r in spanover:
                self.dummyctrl(r, ctrl)
            if e in self.alt_e and with_constraint:
                dv0 = self.dummyctrl(r0, ctrl)
                dv1 = self.dummyctrl(r1, ctrl)
                dv0.constrainer = True
                dv1.constrainer = True
                ctrl[r0 + 1][0].controlled = True
                ctrl[r1 - 1][0].controlled = True

    # iterator that computes all node coordinates and edge routing after
    # just one step (one layer after the other from top to bottom to top).
    # Purely inefficient ! Use it only for "animation" or debugging purpose.
    def draw_step(self):
        ostep = self.ordering_step()
        for s in ostep:
            self.setxy()
            self.draw_edges()
            yield s

    def ordering_step(self, oneway=False):
        self.dirv = -1
        crossings = 0
        for l in self.layers:
            mvmt = l.order()
            crossings += l.ccount
            yield (l, mvmt)
        if oneway or (crossings == 0):
            return
        self.dirv = +1
        while l:
            mvmt = l.order()
            yield (l, mvmt)
            l = l.nextlayer()

    # algorithm by Brandes & Kopf:
    def setxy(self):
        self._edge_inverter()
        self._detect_alignment_conflicts()
        inf = float("infinity")
        # initialize vertex coordinates attributes:
        for l in self.layers:
            for v in l:
                self.grx[v].root = v
                self.grx[v].align = v
                self.grx[v].sink = v
                self.grx[v].shift = inf
                self.grx[v].X = None
                self.grx[v].x = [0.0] * 4
        curvh = self.dirvh  # save current dirvh value
        for dirvh in xrange(4):
            self.dirvh = dirvh
            self._coord_vertical_alignment()
            self._coord_horizontal_compact()
        self.dirvh = curvh  # restore it
        # vertical coordinate assigment of all nodes:
        Y = 0
        for l in self.layers:
            dY = max([v.view.h / 2.0 for v in l])
            for v in l:
                vx = sorted(self.grx[v].x)
                # mean of the 2 medians out of the 4 x-coord computed above:
                avgm = (vx[1] + vx[2]) / 2.0
                # final xy-coordinates :
                v.view.xy = (avgm, Y + dY)
            Y += 2 * dY + self.yspace
        self._edge_inverter()

    # mark conflicts between edges:
    # inner edges are edges between dummy nodes
    # type 0 is regular crossing regular (or sharing vertex)
    # type 1 is inner crossing regular (targeted crossings)
    # type 2 is inner crossing inner (avoided by reduce_crossings phase)
    def _detect_alignment_conflicts(self):
        curvh = self.dirvh  # save current dirvh value
        self.dirvh = 0
        self.conflicts = []
        for L in self.layers:
            last = len(L) - 1
            prev = L.prevlayer()
            if not prev:
                continue
            k0 = 0
            k1_init = len(prev) - 1
            l = 0
            for l1, v in enumerate(L):
                if not self.grx[v].dummy:
                    continue
                if l1 == last or v.inner(-1):
                    k1 = k1_init
                    if v.inner(-1):
                        k1 = self.grx[v.N(-1)[-1]].pos
                    for vl in L[l : l1 + 1]:
                        for vk in L._neighbors(vl):
                            k = self.grx[vk].pos
                            if k < k0 or k > k1:
                                self.conflicts.append((vk, vl))
                    l = l1 + 1
                    k0 = k1
        self.dirvh = curvh  # restore it

    # vertical alignment highly depends on dirh/dirv state.
    def _coord_vertical_alignment(self):
        dirh, dirv = self.dirh, self.dirv
        g = self.grx
        for l in self.layers[::-dirv]:
            if not l.prevlayer():
                continue
            r = None
            for vk in l[::dirh]:
                for m in l._medianindex(vk):
                    # take the median node in dirv layer:
                    um = l.prevlayer()[m]
                    if g[um].dummy and um.controlled and not g[vk].dummy:
                        continue
                    # if vk is "free" align it with um's root
                    if g[vk].align == vk:
                        if dirv == 1:
                            vpair = (vk, um)
                        else:
                            vpair = (um, vk)
                        # if vk<->um link is used for alignment
                        if (vpair not in self.conflicts) and (
                            r == None or dirh * r < dirh * m
                        ):
                            g[um].align = vk
                            g[vk].root = g[um].root
                            g[vk].align = g[vk].root
                            r = m

    def _coord_horizontal_compact(self):
        limit = getrecursionlimit()
        N = len(self.layers) + 10
        if N > limit:
            setrecursionlimit(N)
        dirh, dirv = self.dirh, self.dirv
        g = self.grx
        L = self.layers[::-dirv]
        # recursive placement of blocks:
        for l in L:
            for v in l[::dirh]:
                if g[v].root == v:
                    self.__place_block(v)
        setrecursionlimit(limit)
        # mirror all nodes if right-aligned:
        if dirh == -1:
            for l in L:
                for v in l:
                    x = g[v].X
                    if x:
                        g[v].X = -x
        # then assign x-coord of its root:
        inf = float("infinity")
        rb = inf
        for l in L:
            for v in l[::dirh]:
                g[v].x[self.dirvh] = g[g[v].root].X
                rs = g[g[v].root].sink
                s = g[rs].shift
                if s < inf:
                    g[v].x[self.dirvh] += dirh * s
                rb = min(rb, g[v].x[self.dirvh])
        # normalize to 0, and reinit root/align/sink/shift/X
        for l in self.layers:
            for v in l:
                # g[v].x[dirvh] -= rb
                g[v].root = g[v].align = g[v].sink = v
                g[v].shift = inf
                g[v].X = None

    def __place_block(self, v):
        g = self.grx
        if g[v].X == None:
            # every block is initially placed at x=0
            g[v].X = 0.0
            # place block in which v belongs:
            w = v
            while 1:
                j = g[w].pos - self.dirh  # predecessor in rank must be placed
                r = g[w].rank
                if 0 <= j < len(self.layers[r]):
                    wprec = self.layers[r][j]
                    delta = (
                        self.xspace + (wprec.view.w + w.view.w) / 2.0
                    )  # abs positive minimum displ.
                    # take root and place block:
                    u = g[wprec].root
                    self.__place_block(u)
                    # set sink as sink of prec-block root
                    if g[v].sink == v:
                        g[v].sink = g[u].sink
                    if g[v].sink != g[u].sink:
                        s = g[u].sink
                        newshift = g[v].X - (g[u].X + delta)
                        g[s].shift = min(g[s].shift, newshift)
                    else:
                        g[v].X = max(g[v].X, (g[u].X + delta))
                # take next node to align in block:
                w = g[w].align
                # quit if self aligned
                if w == v:
                    break

    # Basic edge routing applied only for edges with dummy points.
    # Enhanced edge routing can be performed by using the apropriate
    # route_with_ functions from routing.py in the edges' view.
    def draw_edges(self):
        for e in self.g.E():
            if hasattr(e, "view"):
                l = []
                r0, r1 = None, None
                if self.ctrls.has_key(e):
                    D = self.ctrls[e]
                    r0, r1 = self.grx[e.v[0]].rank, self.grx[e.v[1]].rank
                    if r0 < r1:
                        ranks = xrange(r0 + 1, r1)
                    else:
                        if self.ctrls["cons"]:
                            ranks = xrange(r0, r1 - 1, -1)
                        else:
                            ranks = xrange(r0 - 1, r1, -1)
                    l = [D[r][-1].view.xy for r in ranks]
                l.insert(0, e.v[0].view.xy)
                l.append(e.v[1].view.xy)
                if self.ctrls["cons"] and r0 > r1:
                    dy = e.v[0].view.h / 2.0 + self.yspace / 3.0
                    x, y = zip(l[0], l[1])
                    y = max(y) + dy
                    l.insert(1, (x[0], y))
                    l.insert(2, (x[1], y))
                    dy = e.v[1].view.h / 2.0 + self.yspace / 3.0
                    x, y = zip(l[-1], l[-2])
                    y = min(y) - dy
                    l.insert(-1, (x[0], y))
                    l.insert(-2, (x[1], y))
                try:
                    self.route_edge(e, l)
                except AttributeError:
                    pass
                e.view.setpath(l)


#  DIRECTED GRAPH WITH CONSTRAINTS LAYOUT
# ------------------------------------------------------------------------------
class DigcoLayout(object):
    def __init__(self, g):
        # drawing parameters:
        self.xspace = 10
        self.yspace = 10
        self.dr = 10
        self.debug = False

        self.g = g
        self.levels = []
        for i, v in enumerate(self.g.V()):
            assert hasattr(v, "view")
            v.i = i
            self.dr = max((self.dr, v.view.w, v.view.h))
        # solver parameters:
        self._cg_max_iter = g.order()
        self._cg_tolerance = 1.0e-6
        self._eps = 1.0e-5
        self._cv_max_iter = self._cg_max_iter

    def init_all(self, alpha=0.1, beta=0.01):
        y = None
        if self.g.directed:
            # partition g in hierarchical levels:
            y = self.part_to_levels(alpha, beta)
        # initiate positions (y and random in x):
        self.Z = self._xyinit(y)

    def draw(self, N=None):
        if N is None:
            N = self._cv_max_iter
        self.Z = self._optimize(self.Z, limit=N)
        # set view xy from near-optimal coords matrix:
        for v in self.g.V():
            v.view.xy = (self.Z[v.i][0, 0] * self.dr, self.Z[v.i][0, 1] * self.dr)
        self.draw_edges()

    def draw_step(self):
        for x in xrange(self._cv_max_iter):
            self.draw(N=1)
            self.draw_edges()
            yield

    # Basic edge routing with segments
    def draw_edges(self):
        for e in self.g.E():
            if hasattr(e, "view"):
                l = [e.v[0].view.xy, e.v[1].view.xy]
                try:
                    self.route_edge(e, l)
                except AttributeError:
                    pass
                e.view.setpath(l)

    # partition the nodes into levels:
    def part_to_levels(self, alpha, beta):
        opty, err = self.optimal_arrangement()
        ordering = zip(opty, self.g.sV)
        eps = alpha * (opty.max() - opty.min()) / (len(opty) - 1)
        eps = max(beta, eps)
        ordering.sort(reverse=True)
        l = []
        self.levels.append(l)
        for i in xrange(len(ordering) - 1):
            y, v = ordering[i]
            l.append(v)
            v.level = self.levels.index(l)
            if (y - ordering[i + 1][0]) > eps:
                l = []
                self.levels.append(l)
        y, v = ordering[-1]
        l.append(v)
        v.level = self.levels.index(l)
        return opty

    def optimal_arrangement(self):
        b = self.balance()
        y = rand_ortho1(self.g.order())
        return self._conjugate_gradient_L(y, b)

    # balance vector is assembled in finite-element way...
    # this is faster than computing b[i] for each i.
    def balance(self):
        b = array([0.0] * self.g.order(), dtype=float)
        for e in self.g.E():
            s = e.v[0]
            d = e.v[1]
            q = e.w * (self.yspace + (s.view.h + d.view.h) / 2.0)
            b[s.i] += q
            b[d.i] -= q
        return b

    # We compute the solution Y of L.Y = b by conjugate gradient method
    # (L is semi-definite positive so Y is unique and convergence is O(n))
    # note that only arrays are involved here...
    def _conjugate_gradient_L(self, y, b):
        Lii = self.__Lii_()
        r = b - self.__L_pk(Lii, y)
        p = array(r, copy=True)
        rr = sum(r * r)
        for k in xrange(self._cg_max_iter):
            try:
                Lp = self.__L_pk(Lii, p)
                alpha = rr / sum(p * Lp)
                y += alpha / p
                r -= alpha * Lp
                newrr = sum(r * r)
                beta = newrr / rr
                rr = newrr
                if rr < self._cg_tolerance:
                    break
                p = r + beta * p
            except ZeroDivisionError:
                return (None, rr)
        return (y, rr)

    # _xyinit can use diagonally scaled initial vertices positioning to provide
    # better convergence in constrained stress majorization
    def _xyinit(self, y=None):
        if y is None:
            y = rand_ortho1(self.g.order())
        x = rand_ortho1(self.g.order())
        # translate and normalize:
        x = x - x[0]
        y = y - y[0]
        sfactor = 1.0 / max(y.max(), x.max())
        return matrix(zip(x * sfactor, y * sfactor))

    # provide the diagonal of the Laplacian matrix of g
    # the rest of L (sparse!) is already stored in every edges.
    def __Lii_(self):
        Lii = []
        for v in self.g.V():
            Lii.append(sum([e.w for e in v.e]))
        return array(Lii, dtype=float)

    # we don't compute the L.Pk matrix/vector product here since
    # L is sparse (order of |E| not |V|^2 !) so we let each edge
    # contribute to the resulting L.Pk vector in a FE assembly way...
    def __L_pk(self, Lii, pk):
        y = Lii * pk
        for e in self.g.sE:
            i1 = e.v[0].i
            i2 = e.v[1].i
            y[i1] -= e.w * pk[i2]
            y[i2] -= e.w * pk[i1]
        return y

    # conjugate_gradient with given matrix Lw:
    # it is assumed that b is not a multivector,
    # so _cg_Lw should be called in all directions separately.
    # note that everything is a matrix here, (arrays are row vectors only)
    def _cg_Lw(self, Lw, z, b):
        scal = lambda U, V: float(U.transpose() * V)
        r = b - Lw * z
        p = matrix(r, copy=True)
        rr = scal(r, r)
        for k in xrange(self._cg_max_iter):
            if rr < self._cg_tolerance:
                break
            Lp = Lw * p
            alpha = rr / scal(p, Lp)
            z = z + alpha * p
            r = r - alpha * Lp
            newrr = scal(r, r)
            beta = newrr / rr
            rr = newrr
            p = r + beta * p
        return (z, rr)

    def __Dij_(self):
        Dji = []
        for v in self.g.V():
            wd = self.g.dijkstra(v)
            Di = [wd[w] for w in self.g.V()]
            Dji.append(Di)
        # at this point  D is stored by rows,
        # but anymway it's a symmetric matrix
        return matrix(Dji, dtype=float)

    # returns matrix -L^w
    def __Lij_w_(self):
        self.Dij = self.__Dij_()  # we keep D also for L^Z computations
        Lij = self.Dij.copy()
        n = self.g.order()
        for i in xrange(n):
            d = 0
            for j in xrange(n):
                if j == i:
                    continue
                Lij[i, j] = 1.0 / self.Dij[i, j] ** 2
                d += Lij[i, j]
            Lij[i, i] = -d
        return Lij

    # returns vector -L^Z.Z:
    def __Lij_Z_Z(self, Z):
        from math import sqrt

        scal = lambda U, V: float(U.transpose() * V)

        def dist(Zi, Zk):
            v = (Zi - Zk).transpose()
            return sqrt(scal(v, v))

        n = self.g.order()
        # init:
        lzz = Z.copy() * 0.0  # lzz has dim Z (n x 2)
        liz = matrix([0.0] * n)  # liz is a row of L^Z (size n)
        # compute lzz = L^Z.Z while assembling L^Z by row (liz):
        for i in xrange(n):
            iterk_except_i = (k for k in xrange(n) if k != i)
            for k in iterk_except_i:
                liz[0, k] = 1.0 / (self.Dij[i, k] * dist(Z[i], Z[k]))
            liz[0, i] = 0.0  # forced, otherwise next liz.sum() is wrong !
            liz[0, i] = -liz.sum()
            # now that we have the i-th row of L^Z, just dotprod with Z:
            lzz[i] = liz * Z
        return lzz

    def _optimize(self, Z, limit=100):
        scal = lambda U, V: float(U.transpose() * V)
        Lw = self.__Lij_w_()
        K = self.g.order() * (self.g.order() - 1.0) / 2.0
        stress = float("inf")
        count = 0
        deep = 0
        b = self.__Lij_Z_Z(Z)
        while count < limit:
            if self.debug:
                print("count %d" % count)
                print("Z = ", Z)
                print("b = ", b)
            # find next Z by solving Lw.Z = b in every direction:
            x, xerr = self._cg_Lw(Lw[1:, 1:], Z[1:, 0], b[1:, 0])
            y, yerr = self._cg_Lw(Lw[1:, 1:], Z[1:, 1], b[1:, 1])
            Z[1:, 0] = x
            Z[1:, 1] = y
            if self.debug:
                print(" cg -> ")
                print(Z, xerr, yerr)
            # compute new stress:
            FZ = K - float(x.transpose() * b[1:, 0] + y.transpose() * b[1:, 1])
            # precompute new b:
            b = self.__Lij_Z_Z(Z)
            # update new stress:
            FZ += 2 * float(x.transpose() * b[1:, 0] + y.transpose() * b[1:, 1])
            # test convergence:
            print("stress=%.10f" % FZ)
            if stress == 0.0:
                break
            elif abs((stress - FZ) / stress) < self._eps:
                if deep == 2:
                    break
                else:
                    deep += 1
            stress = FZ
            count += 1
        return Z


# ------------------------------------------------------------------------------
class DwyerLayout(object):
    def __init__(self):
        raise NotImplementedError



# Question Answering with Memory using SPA

# Now we will build this model again, using the spa (semantic pointer
# architecture) package built into Nengo 2.0.

# Press the play button to run the simulation.
# Graphs show the colour, shape and cue inputs. The last graph
# shows that the output is most similar to the semantic pointer which was
# initially bound to the given cue. For example, when SQUARE is
# provided as a cue, the output is most similar to BLUE.

import nengo
import nengo.spa as spa
import numpy as np
from nengo.spa import Vocabulary

dim = 32  # The dimensionality of the vectors
rng = np.random.RandomState(4)
vocab = Vocabulary(dimensions=dim, rng=rng, max_similarity=0.1)

# Adding semantic pointers to the vocabulary
CIRCLE = vocab.parse("CIRCLE")
BLUE = vocab.parse("BLUE")
RED = vocab.parse("RED")
SQUARE = vocab.parse("SQUARE")
ZERO = vocab.add("ZERO", [0] * dim)

# Create the spa.SPA network to which we can add SPA objects
model = spa.SPA(label="Question Answering with Memory", vocabs=[vocab])
with model:
    model.A = spa.State(dim)
    model.B = spa.State(dim)
    model.C = spa.State(dim)
    model.D = spa.State(dim)
    model.E = spa.State(dim)
    model.memory = spa.State(dim, feedback=1)

    actions = spa.Actions("D = A * B", "memory = D", "E = memory * ~C")

    model.cortical = spa.Cortical(actions)

    # Function for providing color input
    def color_input(t):
        if t < 0.25:
            return "RED"
        elif t < 0.5:
            return "BLUE"
        return "ZERO"

    # Function for providing shape input
    def shape_input(t):
        if t < 0.25:
            return "CIRCLE"
        elif t < 0.5:
            return "SQUARE"
        return "ZERO"

    # Function for providing the cue
    def cue_input(t):
        if t < 0.5:
            return "ZERO"
        sequence = ["ZERO", "CIRCLE", "RED", "ZERO", "SQUARE", "BLUE"]
        idx = int(((t - 0.5) // (1.0 / len(sequence))) % len(sequence))
        return sequence[idx]

    # Inputs
    model.input = spa.Input(A=color_input, B=shape_input, C=cue_input)



"""One-off functions that create Javascript calls"""

import json

import nengo
import numpy as np

from .static_plots import node_output_plot, response_curve_plot, tuning_curve_plot


def infomodal(ng, uid, **args):
    obj = ng.uids[uid]
    if isinstance(obj, nengo.Ensemble):
        return ensemble_infomodal(ng, uid=uid, **args)
    elif isinstance(obj, nengo.Node):
        return node_infomodal(ng, uid=uid, **args)
    elif isinstance(obj, nengo.Network):
        return net_infomodal(ng, uid=uid, **args)
    else:
        raise NotImplementedError()


def add_modal_title_js(title_text):
    return 'Nengo.modal.title("%s");' % (title_text)


def add_modal_footer_js(footer_text):
    return 'Nengo.modal.footer("%s");' % (footer_text)


def show_modal_js():
    return "Nengo.modal.show();"


def ensemble_infomodal(ng, uid, conn_in_uids, conn_out_uids):
    ens = ng.uids[uid]

    params = [
        (attr, str(getattr(ens, attr)))
        for attr in (
            "n_neurons",
            "dimensions",
            "radius",
            "encoders",
            "intercepts",
            "max_rates",
            "eval_points",
            "n_eval_points",
            "neuron_type",
            "noise",
            "seed",
        )
        if getattr(ens, attr) is not None
    ]

    if ng.page.sim is None:
        plots = "Simulation not yet running. " "Start a simulation to see plots."
    else:
        plots = []
        plots.append(response_curve_plot(ens, ng.page.sim))
        plots.append(tuning_curve_plot(ens, ng.page.sim))

    conninfo = conn_infomodal(ng, uid, conn_in_uids, conn_out_uids)

    js = ["Nengo.modal.title(\"Details for '%s'\");" % ng.page.get_label(ens)]
    js.append('Nengo.modal.footer("close");')
    js.append(
        'Nengo.modal.ensemble_body("%s", %s, %s, %s);'
        % (uid, json.dumps(params), json.dumps(plots), json.dumps(conninfo))
    )
    js.append("Nengo.modal.show();")
    return "\n".join(js)


def node_infomodal(ng, uid, conn_in_uids, conn_out_uids):
    node = ng.uids[uid]

    params = [
        (attr, str(getattr(node, attr)))
        for attr in ("output", "size_in", "size_out")
        if getattr(node, attr) is not None
    ]

    plots = [node_output_plot(node)]

    conninfo = conn_infomodal(ng, uid, conn_in_uids, conn_out_uids)

    js = [add_modal_title_js("Details for '%s'" % (ng.page.get_label(node)))]
    js.append(add_modal_footer_js("close"))
    js.append(
        'Nengo.modal.node_body("%s", %s, %s, %s);'
        % (uid, json.dumps(params), json.dumps(plots), json.dumps(conninfo))
    )
    js.append(show_modal_js())
    return "\n".join(js)


def conn_infomodal(ng, uid, conn_in_uids, conn_out_uids):
    conninfo = {}

    conninfo["obj_type"] = {}
    conninfo["func"] = {}
    conninfo["fan"] = {}

    def get_conn_func_str(conn_uid):
        return (
            str(ng.uids[conn_uid].function)
            if ng.uids[conn_uid].function is not None
            else "Identity function"
        )

    def get_obj_info(nengo_obj):
        if isinstance(nengo_obj, nengo.Node):
            if nengo_obj.output is None:
                return "passthrough", "> 0"
            else:
                return "node", "0"
        elif isinstance(nengo_obj, nengo.Ensemble):
            return "ens", str(nengo_obj.n_neurons)
        elif isinstance(nengo_obj, nengo.Network):
            return "net", str(sum(map(lambda e: e.n_neurons, nengo_obj.all_ensembles)))
        else:
            return "err", "0"

    for conn_uid in conn_in_uids:
        conninfo["obj_type"][conn_uid], conninfo["fan"][conn_uid] = get_obj_info(
            ng.uids[conn_uid].pre_obj
        )
        conninfo["func"][conn_uid] = get_conn_func_str(conn_uid)

    for conn_uid in conn_out_uids:
        conninfo["obj_type"][conn_uid], conninfo["fan"][conn_uid] = get_obj_info(
            ng.uids[conn_uid].post_obj
        )
        conninfo["func"][conn_uid] = get_conn_func_str(conn_uid)

    return conninfo


def net_infomodal(ng, uid, conn_in_uids, conn_out_uids):
    net = ng.uids[uid]

    stats = []
    stats.append(
        {
            "title": "In this network",
            "stats": [
                ("Number of ensembles", len(net.ensembles)),
                ("Total number of neurons", sum(e.n_neurons for e in net.ensembles)),
                ("Number of nodes", len(net.nodes)),
                ("Number of connections", len(net.connections)),
                ("Number of subnetworks", len(net.networks)),
            ],
        }
    )
    stats.append(
        {
            "title": "In this network and subnetworks",
            "stats": [
                ("Number of ensembles", len(net.all_ensembles)),
                (
                    "Total number of neurons",
                    sum(e.n_neurons for e in net.all_ensembles),
                ),
                ("Number of nodes", len(net.all_nodes)),
                ("Number of connections", len(net.all_connections)),
                ("Number of subnetworks", len(net.all_networks)),
            ],
        }
    )

    conninfo = conn_infomodal(ng, uid, conn_in_uids, conn_out_uids)

    js = [add_modal_title_js("Details for '%s'") % (ng.page.get_label(net))]
    js.append(add_modal_footer_js("close"))
    js.append(
        'Nengo.modal.net_body("%s", %s, %s);'
        % (uid, json.dumps(stats), json.dumps(conninfo))
    )
    js.append(show_modal_js())
    return "\n".join(js)
